@@ -14,7 +14,8 @@ from dirspec.basedir import load_config_paths, save_config_path
 from six.moves import input
 
 from .client import (REMOTE_PATTERN, Client, ExpiredTokenIdError,
-                     NoTokenIdError, RemoteError, TokenIdError)
+                     NoTokenIdError, ProtocolVersionError, RemoteError,
+                     TokenIdError, UnfinishedAuthenticationError)
 from .key import PublicKey
 
 
@@ -103,14 +104,21 @@ start.add_argument('-f', '--force',
 def authenticate(args):
     """Authenticate to Geofront server."""
     client = get_client()
-    with client.authenticate() as url:
-        if args.open_browser:
-            print('Continue to authenticate in your web browser...')
-            webbrowser.open(url)
-        else:
-            print('Continue to authenticate in your web browser:')
-            print(url)
+    while True:
+        with client.authenticate() as url:
+            if args.open_browser:
+                print('Continue to authenticate in your web browser...')
+                webbrowser.open(url)
+            else:
+                print('Continue to authenticate in your web browser:')
+                print(url)
         input('Press return to continue')
+        try:
+            client.identity
+        except UnfinishedAuthenticationError as e:
+            print(str(e))
+        else:
+            break
     home = os.path.expanduser('~')
     ssh_dir = os.path.join(home, '.ssh')
     if os.path.isdir(ssh_dir):
@@ -329,4 +337,8 @@ def main(args=None):
                         'Try `{0} authenticate` command.'.format(parser.prog))
         except ExpiredTokenIdError:
             parser.exit('Authentication renewal required.\n'
-                        'Try `{0} authenticate` command.')
+                        'Try `{0} authenticate` command.'.format(parser.prog))
+        except ProtocolVersionError as e:
+            parser.exit('geofront-cli seems incompatible with the server.\n'
+                        'Try `pip install --upgrade geofront-cli` command.\n'
+                        'The server version is {0}.'.format(e.server_version))
